library(stockassessment)

load("run/model.RData")
######################################
#f_mean <- mean(fbar(ple7a)[, ac(2019:2021)])
f_mean <- mean(tail(fbartable(fit)[,"Estimate"],3))
#f_mean <- fbar(ple7a)[, ac(2021)] 
F_MSY <- 0.196
F_MSY_lower <- 0.133
F_MSY_upper <- 0.293
Fpa <- 0.403 #Fp05 #Ppa = 0.355 last year 2020
Flim <- 0.495
Blim <- 3958
Bpa <- 5294
MSY_Btrigger <- 8757

prev.TAC <- 2747
prev.advice <- 2925

logobs <- cbind(as.data.frame(fit$data$aux), logobs = fit$data$logobs)
### remove log scale
logobs$obs <- exp(logobs$logobs)

#Extra line added in 2021 because there  were NAs in the observations from the missing 2020 survey:
logobs <- subset(logobs, fleet==1) 
### remove columns not required anymore
catch <- logobs[, c("year", "age", "obs")]
catchL <- catch
catch <- data.frame(year=unique(catchL$year))
for (a in unique(catchL$age)) catch <- cbind(catch, catchL$obs[catchL$age==a])
colnames(catch) <- c("year", unique(catchL$age))

#catch <- cast(catch, year ~ age, value = "obs", fun.aggregate = sum)
### remove SSB column (age -1), if available
catch <- catch[, setdiff(names(catch), "-1")]
### remove year column and set year as row name

row.names(catch) <- catch$year
catch <- catch[, setdiff(names(catch), "year")]
### replace NAs with 0
catch[is.na(catch)] <- 0


assessment_summary <- as.data.frame(summary(fit))
assessment_summary$deadcatch <- rowSums(catch*fit$data$catchMeanWeight)
assessment_summary$ICESl <- rowSums(catch*fit$data$landFrac*fit$data$landMeanWeight)
assessment_summary$ICESd <- rowSums(catch*(1-fit$data$landFrac)*fit$data$disMeanWeight)/0.6
assessment_summary$ICESd_d <- assessment_summary$ICESd * 0.6    #Dead discards
assessment_summary$ICESd_l <- assessment_summary$ICESd * 0.4    #Live discards

###########################################
deaddisrate <- mean(tail(assessment_summary$ICESd_d / assessment_summary$deadcatch, 3))   ##Proportion of the dead catch that is landed # mean of the last 3 years??



#########  forecast   ##########

FC<-list()

# 2021F=Fsq then Fmsy
set.seed(12345) #sets the starting number used to generate a sequence of random numbers - it ensures that you get the same result if you start with that same seed each time you run the same process
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,F_MSY,F_MSY), 
                               label="2022F=Fsq then Fmsy", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)
               
                
# 2021F=Fsq then Fmsy lower = 0.133
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,F_MSY_lower,F_MSY_lower), 
                               label="2022F=Fsq then Fmsy lower", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)


# 2021F=Fsq then Fmsy upper = 0.293
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,F_MSY_upper,F_MSY_upper), 
                               label="2022F=Fsq then Fmsy upper", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)


# 2021F=Fsq then zero
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,0,0), fval=c(NA,f_mean,NA,NA),
                               label="2022F=Fsq then zero", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)


# 2021F=Fsq then Fpa = 0.355
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,Fpa,Fpa), 
                               label="2022F=Fsq Fpa", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)


# 2021F=Fsq then Flim = 0.495
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,Flim,Flim), 
                               label="2022F=Fsq then Flim",rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)


#F that will get us to Blim
optim_B <- function(f2target, target, f_mean, F_MSY, B_yr,
                    ave.years, rec.years){
  set.seed(12345)
  f1 <- forecast(fit, 
                 fscale =   c(1,          NA,       NA, NA), 
                 fval =     c(NA, f_mean, f2target, F_MSY), 
                 catchval = c(NA,         NA,       NA, NA),
                 ave.years = ave.years, 
                 rec.years = rec.years, savesim = TRUE)####
  ssb <- attr(f1,"shorttab")["ssb", as.character(B_yr)]
  out <- 1E+3*(ssb - target)^2
  return(out)
}
f_val <- optimise(f = optim_B, interval = seq(from = 0, to = 3, by = 0.001), 
                  target = Blim, f_mean = f_mean, F_MSY = F_MSY,
                  B_yr = 2024, 
                  ave.years = max(fit$data$years) + (-2:0), 
                  rec.years = 2015:2021)$minimum
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale = c(1, NA, NA, 1), 
                               fval = c(NA, f_mean, f_val, NA), 
                               ave.years = max(fit$data$years) + (-2:0), 
                               rec.years = 2015:2021,
                               label="Fsq, SSB(2024)=Blim", splitLD=TRUE, savesim = TRUE)


#F that will get us to Bpa
f_val <- optimise(f = optim_B, interval = seq(from = 0, to = 3, by = 0.001), 
                  target = Bpa, f_mean = f_mean, F_MSY = F_MSY,
                  B_yr = 2024, 
                  ave.years = max(fit$data$years) + (-2:0), 
                  rec.years = 2015:2021)$minimum
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale = c(1, NA, NA, 1), 
                               fval = c(NA, f_mean, f_val, NA), 
                               ave.years = max(fit$data$years) + (-2:0), 
                               rec.years = 2015:2021,
                               label="Fsq, SSB(2024)=Bpa", splitLD=TRUE, savesim = TRUE)


#F that will get us to MSY Btrigger
f_val <- optimise(f = optim_B, interval = seq(from = 0, to = 3, by = 0.001), 
                  target = MSY_Btrigger, f_mean = f_mean, F_MSY = F_MSY,
                  B_yr = 2024, 
                  ave.years = max(fit$data$years) + (-2:0), 
                  rec.years = 2015:2021)$minimum
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale = c(1, NA, NA, 1), 
                               fval = c(NA, f_mean, f_val, NA), 
                               ave.years = max(fit$data$years) + (-2:0), 
                               rec.years = 2015:2021,
                               label="Fsq, SSB(2024)=MSY Btrigger", splitLD=TRUE, savesim = TRUE)


# SQ
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,f_mean,f_mean), 
                               label="2022F=Fsq then Fsq", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)





#Rollover TAC
TAC_red <- prev.TAC /  ((1-deaddisrate) + deaddisrate / 0.6)
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,NA,NA),
                               catchval=c(NA,NA,TAC_red, TAC_red),
                               label="2022F=Fsq then TAC", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)




#Rollover advice
advice_red <- prev.advice /  ((1-deaddisrate) + deaddisrate / 0.6)
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(1,NA,NA,NA), fval=c(NA,f_mean,NA,NA),
                               catchval=c(NA,NA,advice_red, advice_red),
                               label="2022F=Fsq then 2022 advice", rec.years=2015:2021,
                               ave.years = max(fit$data$years) + (-2:0),
                               splitLD = TRUE, savesim = TRUE)


#F that will get us to MSY Btrigger

B_target <- median(FC[[1]][[3]]$ssb    )  ##SSB at the start of advice year
f_val <- optimise(f = optim_B, interval = seq(from = 0, to = 3, by = 0.001), 
                  target = B_target, f_mean = f_mean, F_MSY = F_MSY,
                  B_yr = 2024, 
                  ave.years = max(fit$data$years) + (-2:0), 
                  rec.years = 2015:2021)$minimum
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale = c(1, NA, NA, 1), 
                               fval = c(NA, f_mean, f_val, NA), 
                               ave.years = max(fit$data$years) + (-2:0), 
                               rec.years = 2015:2021,
                               label="Fsq, SSB(2024)=SSB(2023)", splitLD=TRUE, savesim = TRUE)


save(FC, file="run/forecast.RData")
save(deaddisrate, file="run/forecast1.RData")
save(prev.TAC, file="run/forecast2.RData")
save(prev.advice, file="run/forecast3.RData")

